==> org-lsst-ccs-subsystem-utility-main-1.4.4-sources.jar: org/lsst/ccs/subsystem/utility/BfrDeviceUT.java <==
package org.lsst.ccs.subsystem.utility;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.drivers.auxelex.Bfr;
import org.lsst.ccs.drivers.commons.DriverException;
import org.lsst.ccs.monitor.Channel;
import org.lsst.ccs.monitor.Device;
import org.lsst.ccs.subsystem.common.ErrorUtils;
import org.lsst.ccs.subsystem.utility.constants.UtilityConfig;
import org.lsst.ccs.subsystem.utility.constants.UtilTrunkSwitches;

/**
 *  Interface to the SLAC BFR.
 *
 *  @author Owen Saxton
 */
public class BfrDeviceUT extends Device implements SwitchControl {

    /**
     *  Constants.
     */
    public static final int
        CHAN_HEATER_1    = Bfr.RELAY_HTR_1,
        CHAN_HEATER_2    = Bfr.RELAY_HTR_2;

    private static final String
        NODE   = "node";
    private static final Logger LOG = Logger.getLogger(BfrDeviceUT.class.getName());
    private boolean initError = false;


    /**
     *  Data fields.
     */
    @ConfigurationParameter(name=NODE, category=UtilityConfig.CATEGORY_UTIL, isFinal=true)
    private volatile Integer node;

    private final Bfr bfr = new Bfr();
    private double currents[] = null;


    /**
     *  Performs configuration.
     */
    @Override
    protected void initDevice() {
        if (node == null) {
            ErrorUtils.reportConfigError(LOG, name, NODE, "is missing");
        }
        fullName = path + " (BFR board at node " + node + ")";
    }


    /**
     *  Performs full initialization.
     */
    @Override
    public void initialize()
    {
        try {
            bfr.open(node, Bfr.PORT_1);
            initSensors();
            LOG.log(Level.INFO, "Connected to {0}", fullName);
            initError = false;
            setOnline(true);
        }
        catch (DriverException e) {
            if (!initError) {
                LOG.log(Level.SEVERE, "Error connecting to {0}: {1}", new Object[]{fullName, e});
                initError = true;
            }
        }
    }


    /**
     *  Closes the connection.
     */
    @Override
    protected void close()
    {
        try {
            bfr.close();
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error disconnecting from {0}: {1}", new Object[]{fullName, e});
        }
    }


    /**
     *  Checks a monitoring channel's parameters for validity.
     *
     *  @param  ch  The channel to check
     *  @return  Two-element array containing the encoded type [0] and subtype [1] 
     *  @throws  Exception  If parameters are invalid
     */
    @Override
    protected int[] checkChannel(Channel ch) throws Exception
    {
        int hwChan = ch.getHwChan();
        if (hwChan < 0 || hwChan >= Bfr.NUM_SENSORS) {
            ErrorUtils.reportChannelError(LOG, ch.getPath(), "hw channel number", hwChan);
        }
        return new int[]{0, 0};
    }


    /**
     *  Gets the group for a channel.
     *
     *  @param  ch  The channel
     *  @return  The group name, or null if not in a group
     */
    @Override
    protected String getGroupForChannel(Channel ch) {
        return "currents";
    }


    /**
     *  Reads all monitored channels.
     * 
     * @param group The group to read.
     *
     */
    @Override
    protected void readChannelGroup(String group)
    {
        currents = null;
        if (isOnline()) {
            try {
                currents = bfr.readCurrent(Bfr.CurrentType.AC);
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error reading {0} currents: {1}", new Object[]{fullName, e});
                setOnline(false);
            }
        }
    }


    /**
     *  Reads a monitored channel.
     *
     *  @param  ch  The channel to read
     *  @return  The read value
     */
    @Override
    protected double readChannel(Channel ch)
    {
        return currents == null ? Double.NaN : currents[ch.getHwChan()];
    }


    /**
     *  Gets the switch device type.
     * 
     *  @return  The switch device type enumeration
     */
    @Override
    public int getSwitchDevice()
    {
        return UtilTrunkSwitches.DEVC_BFR;
    }


    /**
     *  Turns a relay on.
     *
     *  @param  relay  The relay number
     */
    @Override
    public void switchOn(int relay)
    {
        try {
            bfr.setRelayOn(relay);
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error setting relay {0} on {1}: {2}", new Object[]{relay, fullName, e});
        }
        
    }


    /**
     *  Turns a relay off.
     *
     *  @param  relay  The relay number
     */
    @Override
    public void switchOff(int relay)
    {
        try {
            bfr.setRelayOff(relay);
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error setting relay {0} on {1}: {2}", new Object[]{relay, fullName, e});
        }
        
    }


    /**
     *  Gets the state of a relay.
     *
     *  @param  relay  The relay number
     *  @return  Whether the relay is on or off
     */
    @Override
    public Boolean isSwitchOn(int relay)
    {
        if (!isOnline() || relay < 0) return null;
        try {
            return bfr.isRelayOn(relay);
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error getting state for relay {0} on {1}: {2}", new Object[]{relay, fullName, e});
            return null;
        }
    }

}

==> org-lsst-ccs-subsystem-utility-main-1.4.14-sources.jar: org/lsst/ccs/subsystem/utility/BfrDeviceUT.java <==
package org.lsst.ccs.subsystem.utility;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.drivers.auxelex.Bfr;
import org.lsst.ccs.drivers.commons.DriverException;
import org.lsst.ccs.monitor.Channel;
import org.lsst.ccs.monitor.Device;
import org.lsst.ccs.subsystem.common.ErrorUtils;
import org.lsst.ccs.subsystem.utility.constants.UtilityConfig;
import org.lsst.ccs.subsystem.utility.constants.UtilTrunkSwitches;

/**
 *  Interface to the SLAC BFR.
 *
 *  @author Owen Saxton
 */
public class BfrDeviceUT extends Device implements SwitchControl {

    /**
     *  Constants.
     */
    public static final int
        CHAN_HEATER_1    = Bfr.RELAY_HTR_1,
        CHAN_HEATER_2    = Bfr.RELAY_HTR_2;

    private static final String
        NODE   = "node";
    private static final Logger LOG = Logger.getLogger(BfrDeviceUT.class.getName());
    private boolean initError = false;


    /**
     *  Data fields.
     */
    @ConfigurationParameter(category=UtilityConfig.CATEGORY_UTIL, isFinal=true, name=NODE, units="unitless", description="BFR node number")
    private volatile Integer node;

    private final Bfr bfr = new Bfr();
    private double currents[] = null;


    /**
     *  Performs configuration.
     */
    @Override
    protected void initDevice() {
        if (node == null) {
            ErrorUtils.reportConfigError(LOG, name, NODE, "is missing");
        }
        fullName = path + " (BFR board at node " + node + ")";
    }


    /**
     *  Performs full initialization.
     */
    @Override
    public void initialize()
    {
        try {
            bfr.open(node, Bfr.PORT_1);
            initSensors();
            LOG.log(Level.INFO, "Connected to {0}", fullName);
            initError = false;
            setOnline(true);
        }
        catch (DriverException e) {
            if (!initError) {
                LOG.log(Level.SEVERE, "Error connecting to {0}: {1}", new Object[]{fullName, e});
                initError = true;
            }
        }
    }


    /**
     *  Closes the connection.
     */
    @Override
    protected void close()
    {
        try {
            bfr.close();
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error disconnecting from {0}: {1}", new Object[]{fullName, e});
        }
    }


    /**
     *  Checks a monitoring channel's parameters for validity.
     *
     *  @param  ch  The channel to check
     *  @return  Two-element array containing the encoded type [0] and subtype [1] 
     *  @throws  Exception  If parameters are invalid
     */
    @Override
    protected int[] checkChannel(Channel ch) throws Exception
    {
        int hwChan = ch.getHwChan();
        if (hwChan < 0 || hwChan >= Bfr.NUM_SENSORS) {
            ErrorUtils.reportChannelError(LOG, ch.getPath(), "hw channel number", hwChan);
        }
        return new int[]{0, 0};
    }


    /**
     *  Gets the group for a channel.
     *
     *  @param  ch  The channel
     *  @return  The group name, or null if not in a group
     */
    @Override
    protected String getGroupForChannel(Channel ch) {
        return "currents";
    }


    /**
     *  Reads all monitored channels.
     * 
     * @param group The group to read.
     *
     */
    @Override
    protected void readChannelGroup(String group)
    {
        currents = null;
        if (isOnline()) {
            try {
                currents = bfr.readCurrent(Bfr.CurrentType.AC);
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error reading {0} currents: {1}", new Object[]{fullName, e});
                setOnline(false);
            }
        }
    }


    /**
     *  Reads a monitored channel.
     *
     *  @param  ch  The channel to read
     *  @return  The read value
     */
    @Override
    protected double readChannel(Channel ch)
    {
        return currents == null ? Double.NaN : currents[ch.getHwChan()];
    }


    /**
     *  Gets the switch device type.
     * 
     *  @return  The switch device type enumeration
     */
    @Override
    public int getSwitchDevice()
    {
        return UtilTrunkSwitches.DEVC_BFR;
    }


    /**
     *  Turns a relay on.
     *
     *  @param  relay  The relay number
     */
    @Override
    public void switchOn(int relay)
    {
        try {
            bfr.setRelayOn(relay);
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error setting relay {0} on {1}: {2}", new Object[]{relay, fullName, e});
        }
        
    }


    /**
     *  Turns a relay off.
     *
     *  @param  relay  The relay number
     */
    @Override
    public void switchOff(int relay)
    {
        try {
            bfr.setRelayOff(relay);
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error setting relay {0} on {1}: {2}", new Object[]{relay, fullName, e});
        }
        
    }


    /**
     *  Gets the state of a relay.
     *
     *  @param  relay  The relay number
     *  @return  Whether the relay is on or off
     */
    @Override
    public Boolean isSwitchOn(int relay)
    {
        if (!isOnline() || relay < 0) return null;
        try {
            return bfr.isRelayOn(relay);
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error getting state for relay {0} on {1}: {2}", new Object[]{relay, fullName, e});
            return null;
        }
    }

}
